# -*- coding: utf-8 -*-
""" Cell type representing the cells in Bion with RBG values
"""

__author__ = "Michael Vo <vomichae@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.cell.Cell import *
import random

class BionColorCellBC(Cell):
    """
    This is a simple Cell type with RBG values and simple cooperation.

    Configuration: There are no parameters configurable for this Cell type.


	red, green, and blue are rbg values from 0 to 255
	energy is needed to reproduce
	public good is produced each update if the cell has gene for it turned on
	"""
	
    red = 0
    green = 0
    blue = 0
    
    energy_level = 0
    public_good_level = 0
    public_good_gene = False

    def __init__(self, world, topology, node, id, type=-1):
        """Initialize a RPSCell object

        The type for the cell is selected at random.

        Parameters:

        *world*
            A reference to the World
        *topology*
            A reference to the topology in which the Cell will reside
        *node*
            A reference to the node on which the Cell resides
        *id*
            A unique ID for the cell
        *type*
            The type of cell to initialize (-1 for random)

        """

        Cell.__init__(self,world,topology,node,id)
        self.type = 1
            
        """ random rgb value """
        self.red = random.randint(0,255)   
        self.green = random.randint(0,255)
        self.blue = random.randint(0,255)
        
        self.topology.increment_type_count(self.type)
        
        """ Read in parameters """
        self.mutation_rate = self.world.config.getfloat('Bion', 'mutation_rate', .005)
        self.mutation_sigma = self.world.config.getfloat('Bion', 'mutation_sigma', 0.05)

        self.visit_bonus = self.world.config.getfloat('Bion', 'visit_bonus', 4)
        self.update_energy = self.world.config.getfloat('Bion', 'update_energy', .0625)
        self.child_gift = self.world.config.getfloat('Bion', 'child_gift', 0)
        
        self.public_good_cost = self.world.config.getfloat('Bion', 'public_good_cost', .5)
        self.public_good_benefit = self.world.config.getfloat('Bion', 'public_good_benefit', 3)
        self.public_good_threshold = self.world.config.getfloat('Bion', 'public_good_threshold', 5)
        self.public_good_loss = self.world.config.getfloat('Bion', 'public_good_loss', 0.01)
        
        """ default public gene to false """
        self.public_good_gene=False
        	
    def __str__(self):
        """Produce a string to be used when the object is printed"""
        return 'BionColorCellBC %d Red %d Green %d Blue %d' % (self.id, self.red, self.green, self.blue)

    def type(self):
        """Return the name of the type of this cell"""
        return self.type
        return 'Red %d Green %d Blue %d' % (self.red, self.green, self.blue)

    def update(self, neighbors):
        """Update the cell by creating public good and reproducing

        Parameters:

        *neighbors*
            A list of neighboring cells

        """

        """ find total public good in vicinity, if greater than threshold give benefit """
        total_public_good = self.public_good_level
        for n in neighbors:
        	total_public_good += n.public_good_level
        
        """ give cell update energy """
        if total_public_good >= self.public_good_threshold:
        	self.energy_level += self.update_energy*self.public_good_benefit
        else:
            self.energy_level += self.update_energy
        
        """ if this cell is cooperator, produce public good at cost """
        if self.public_good_gene:
        	self.energy_level -= self.update_energy*self.public_good_cost
        	self.public_good_level += 1
        
        # Reproduce if energy is sufficient
        if self.energy_level >= 1:
            child = random.choice(neighbors)

            # Mutate red, green, and blue alleles independently
            if random.random() < self.mutation_rate:
                child.red = int(round(random.normalvariate(self.red, self.mutation_sigma)))
            if random.random() < self.mutation_rate:
                child.green = int(round(random.normalvariate(self.green, self.mutation_sigma)))
            if random.random() < self.mutation_rate:
                child.blue = int(round(random.normalvariate(self.blue, self.mutation_sigma)))

            # Mutate the public good gene
            if random.random() < self.mutation_rate:
                child.public_good_gene = not self.public_good_gene

            # Donate some energy to the child.  remove from self.
            donation = self.energy_level * self.child_gift
            child.energy_level = donation
            self.energy_level -= donation
        	
            # Deduct energy used to reproduce
            self.energy_level-=1
        
        # Remove some of the public good
        self.public_good_level -= self.public_good_loss
    
    def visit(self, frac=1):
    	self.energy_level += (self.visit_bonus * frac)




# -*- coding: utf-8 -*-
"""
Well-mixed topology in which each Cell is a neighbor of every other Cell.

This topology is modeled as a graph with no edges between the nodes.  Although
a complete graph would perhaps model a well-mixed topology more accurately,
this would prohibit large populations due to the associated memory
requirements.

Cells are given a randomly-assigned location to more easily allow for
visualization.  It should be noted that in this topology, interactions
are not localized, so the neighbors with which a cell interacts will
be located throughout the environment.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import random
import math

from seeds.CellManager import *
from seeds.topology.Topology import *
from seeds.Resource import *
from seeds.ResourceManager import *

import networkx as nx

class WellMixedTopology(Topology):
    """
    Well-mixed topology with a configurable number of interactions

    Configuration: All configuration options should be specified in the
        WellMixedTopology block.

        size
            Total number of cells in the population
        num_interactions
            When a Cell is updated, a list of neighbors is passed to it.
            This parameter specifies the size of a random subset of cells
            to be used as this neighbor set.  By default, all cells in
            the populaion are passed as neighbors.

    Example:
        [WellMixedTopology]
        size = 100000
        num_interactions = 10


    """
    def __init__(self, world, id):
        """Initialize a WellMixedTopology object"""
        Topology.__init__(self, world, id)

        self.size = self.world.config.getint(section='WellMixedTopology',
                                             name='size')

        self.num_interactions = self.world.config.getint(section='WellMixedTopology',
                                                         name='num_interactions',
                                                         default=self.size)

        if self.size < 1:
            print 'Error: Must specify the size of the population'

        self.graph = nx.empty_graph()
        self.graph.name = "well_mixed_graph"
        self.graph.add_nodes_from(range(self.size))

        for n in self.graph.nodes():
            self.graph.node[n]['cell'] = self.cell_manager.newcell(n, n)
            self.graph.node[n]['cell'].coords = (random.random(),random.random())
            self.graph.node[n]['resource_manager'] = ResourceManager(world, self)

    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return 'Well-Mixed Topology (%d cells, %d interactions)' % (self.size, self.num_interactions)


    def get_neighbors(self, node):
        """Get a randomly-selected list of neighboring cells for a given node

        Parameters:

        *node*
            The ID of the node whose neighboring cells to get

        """
        neighbors = random.sample(self.graph.nodes(), self.num_interactions)
        return [self.graph.node[n]['cell'] for n in neighbors]




# -*- coding: utf-8 -*-
""" Cell type representing the cells in Bion with RBG values
"""

__author__ = "Michael Vo <vomichae@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.cell.Cell import *
import random

class BionColorCell(Cell):
    """
    This is a simple Cell type with RBG values and simple cooperation.

    Configuration: There are no parameters configurable for this Cell type.


	red, green, and blue are rbg values from 0 to 255
	energy is needed to reproduce
	public good is produced each update if the cell has gene for it turned on
	"""
	
    red = 0
    green = 0
    blue = 0
    
    energy_level = 0
    public_good_level = 0
    public_good_gene = False

    def __init__(self, world, topology, node, id, type=-1):
        """Initialize a RPSCell object

        The type for the cell is selected at random.

        Parameters:

        *world*
            A reference to the World
        *topology*
            A reference to the topology in which the Cell will reside
        *node*
            A reference to the node on which the Cell resides
        *id*
            A unique ID for the cell
        *type*
            The type of cell to initialize (-1 for random)

        """

        Cell.__init__(self,world,topology,node,id)
        
        """ dummy cell type"""
        if type == -1:
        	self.type = 1
        else:
            self.type = 1
            
        """ random rgb value """
        self.red = random.randint(0,255)   
        self.green = random.randint(0,255)
        self.blue = random.randint(0,255)
        
        self.topology.increment_type_count(self.type)
        
        """ Read in parameters """
        self.mutation_rate = self.world.config.getfloat('Bion', 'mutation_rate', .05)
        self.visit_bonus = self.world.config.getfloat('Bion', 'visit_bonus', 4)
        self.update_energy = self.world.config.getfloat('Bion', 'update_energy', .0625)
        self.child_gift = self.world.config.getfloat('Bion', 'child_gift', 0)
        
        self.public_good_cost = self.world.config.getfloat('Bion', 'public_good_cost', .5)
        self.public_good_benefit = self.world.config.getfloat('Bion', 'public_good_benefit', 3)
        self.public_good_threshold = self.world.config.getfloat('Bion', 'public_good_threshold', 5)
        
        """ default public gene to false """
        self.public_good_gene=False
        	
    def __str__(self):
        """Produce a string to be used when the object is printed"""
        return 'RPCell %d Red %d Green %d Blue %d' % (self.id, self.red, self.green, self.blue)

    def type(self):
        """Return the name of the type of this cell"""
        return 'Red %d Green %d Blue %d' % (self.red, self.green, self.blue)

    def update(self, neighbors):
        """Update the cell by creating public good and reproducing

        Parameters:

        *neighbors*
            A list of neighboring cells

        """
        
        """ give cell update energy """
        self.energy_level+=self.update_energy
        
        """ if this cell is cooperator, produce public good at cost """
        if self.public_good_gene==True:
        	self.energy_level-=self.update_energy*self.public_good_cost
        	self.public_good_level+=1
        
        """ find total public good in vicinity, if greater than threshold give benefit """
        
        total_public_good = self.public_good_level
        for n in neighbors:
        	total_public_good+=n.public_good_level
        
        if(total_public_good>=self.public_good_threshold):
        	self.energy_level+=self.update_energy*self.public_good_benefit
        	
        # Pick a random neighbor to reproduce into if it comes down to it
        child = random.choice(neighbors)
        
        
        
        if self.energy_level>=1:
        	
        	# Possible modification of colors is mutation rate times 255
        	range = int(self.mutation_rate*255)
        	
        	# modify colors with mutation (not quite accurate for polygenic
        	# since it should be mut. rate for each of 255 genes. fix later)
        	child.blue = self.blue+random.randint(-range,range)
        	child.red = self.red+random.randint(-range,range)
        	child.green = self.green+random.randint(-range,range)
        	
        	# if there's a gift for new children, give it here
        	child.energy_level = self.energy_level*self.child_gift
        	
        	# public good level remains the same... still left in cell
        	child.public_good_level = child.public_good_level
        	
        	# see if we change the public good gene
        	if random.randrange(0,1000)>self.mutation_rate*1000:
        		child.public_good_gene = self.public_good_gene
        	else:
        		child.public_good_gene = not self.public_good_gene
        		
        	self.energy_level-=1
        
        # if public good level is greater than 0, subtract from it
        if self.public_good_level>0:
        	self.public_good_level-=.5
    
    def visit(self):
    	self.energy_level+=self.visit_bonus



# -*- coding: utf-8 -*-
"""
Stop the experiment when the number of existing Cell types drops below a
threshold in all populations.  This allows experiments to end earlier
than the configured number if epochs have passed or experiments to not
specify the total number of epochs to observe.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import csv

from seeds.Action import *

class StopOnConvergence(Action):
    """ Stop the experiment when the number of exisiting cell types drops below
    the configured threshold in all populations

    Config file settings: All parameters for this action are specified in the
    [StopOnConvergence] configuration block.

    [StopOnConvergence]
    threshold = 3       Number of types below which the run will stop.
    epoch_start = 3     Epoch at which to start checking (default 0)
    epoch_end = 100     Epoch at which to stop checking (default end of experiment)
    frequency = 2       Frequency (epochs) to check the populations.  In this example, we write every other epoch.  (default 1)

    """
    def __init__(self, world):
        """Initialize the StopOnConvergence Action"""
        Action.__init__(self, world)

        self.threshold = self.world.config.getint('StopOnConvergence', 'threshold', 0)
        self.epoch_start = self.world.config.getint('StopOnConvergence', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('StopOnConvergence', 'epoch_end',
                                                  default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('StopOnConvergence', 'frequency', 1)
        self.filename = self.world.config.get('StopOnConvergence', 'filename', 'cell_locations')
        self.name = "StopOnConvergence"

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        converged = True

        for top in self.world.topology_manager.topologies:
            num_active_types = 0

            for tcount in top.typeCount:
                if tcount > 0:
                    num_active_types += 1

            if num_active_types >= self.threshold:
                converged = False

        if converged:
            self.world.end()




# -*- coding: utf-8 -*-
"""
The World encompasses all aspects of the experiment. It maintains the actions,
the topologies (Cells), the configuration, and time.

The state of the World can be saved or loaded using Snapshots.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import random
import time

from seeds.ActionManager import *
from seeds.Config import *
from seeds.PluginManager import *
from seeds.Snapshot import *
from seeds.TopologyManager import *

class World(object):
    """
    The World object captures the state of a experiment

    Properties:

    config
        A Config object storing the configuration for the experiment
    plugin_manager
        A PluginManager object which manages all Plugins for the experiment
    topology_manager
        A TopologyManager object which manages all Topologies in the experiment
    action_manager
        An ActionManager object which manages all Actions in the experiment
    epoch
        An integer storing the current epoch (unit of time)
    proceed
        Boolean value indicating whether or not the experiment should continue.

    """

    def __init__(self, configfile=None, seed=-1):
        """Initialize a World object

        Parameters:

        *configfile*
            Name of configuration file for the experiment.  If none is
            provided, the experiment will either use defaults for all
            parameters or values provided to the Config object elsewhere (e.g.,
            through the command-line or GUI program).
        *seed*
            Seed for pseudorandom number generator.  If undefined, the current
            time will be used.

        """

        self.config = Config(configfile)
        self.epoch = 0
        self.proceed = True
        self.seed = seed
        self.is_setup = False


    def setup(self):
        """Set up the World including its Actions, Topologies, and Cells"""
        if self.seed == -1:
            configseed = self.config.getint('Experiment', 'seed', default=-1)
            if configseed != -1:
                self.seed = configseed
            else:
                self.seed = int(time.time()*10)

        random.seed(self.seed)
        self.config.set('Experiment', 'seed', self.seed)

        self.experiment_epochs = self.config.getint('Experiment', 'epochs',
                                                    default=-1)

        self.plugin_manager = PluginManager(self)
        self.topology_manager = TopologyManager(self)
        self.action_manager = ActionManager(self)

        self.is_setup = True

    def update(self):
        """Update the World and all of its objects"""
        if not self.is_setup:
            self.setup()

        self.action_manager.update()	# Update the actions
        self.topology_manager.update()	# Update the topologies/cells
        self.epoch += 1

        # If we've surpassed the configured number of epochs to run for, set
        # proceed to false
        if self.experiment_epochs != -1 and self.epoch > self.experiment_epochs:
            self.proceed = False

    def end(self):
        """Set the experiment to end after this epoch"""
        self.proceed = False

    def get_snapshot(self):
        """Get a Snapshot containing the state of the World"""
        s = Snapshot()
        s.update(self)
        return s

    def load_snapshot(self, filename):
        """Load a Snapshot from file and set the state of the World

        Parameters:

        *filename*
            The file from which to load the Snapshot

        """

        print "NOTICE: Loading snapshots is currently not working!"

        s = Snapshot() 
        s.read(filename)
        s.apply(self)




# -*- coding: utf-8 -*-
"""
Print information about the amount and distribution of resource present in each
population
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import csv

from seeds.Action import *


class PrintResourceStats(Action):
    """ Print information about the levels of resource present in each Topology

        Config file settings:
        [PrintResourceStats]
        epoch_start = 3     Epoch at which to start writing (default 0)
        epoch_end = 100     Epoch at which to stop writing (default end of experiment)
        frequency = 2       Frequency (epochs) to write.  In this example, we write every other epoch.  (default 1)
        resource = glucose  Name of resource
        filename = virulence.dat  Filename to be written to

    """

    def __init__(self, world):
        """Initialize the PrintResourceStats instance"""
        Action.__init__(self, world)

        self.epoch_start = self.world.config.getint('PrintResourceStats', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('PrintResourceStats', 'epoch_end', default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('PrintResourceStats', 'frequency', 1)
        self.resource = self.world.config.get('PrintResourceStats', 'resource')
        self.filename = self.world.config.get('PrintResourceStats', 'filename', 'resources.dat')

        data_file = self.datafile_path(self.filename)
        self.writer = csv.writer(open(data_file, 'w'))

        self.writer.writerow(['#epoch','population','mean resource level','std resource level'])

    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return 'PrintResourceStats Object (epoch start: %d)(epoch end: %d)(frequency: %d)' % (self.epoch_start, self.epoch_end, self.frequency)

    def update(self):
        """Execute the action"""
        if self.skip_update():
	        return

        reslevels = []

        for top in self.world.topology_manager.topologies:
            for n in top.graph.nodes():
                r = top.graph.node[n]['resource_manager'].get_resource(self.resource)
                if r != None:
                    reslevels.append(r.level)

            if len(reslevels) > 0:
                resmean = self.mean(reslevels)
                resstd = self.std(reslevels)
            else:
                resmean = 0
                resstd = 0

            row = [self.world.epoch] + [top.id, resmean, resstd]
            self.writer.writerow(row)

    def mean(self, data):
        """Calculate the mean of a list of numbers

        Parameters:

        *data*
            a list of numbers whose mean to calculate

        """

        return float(sum(data))/len(data)

    def std(self, data):
        """Calculate the standard deviation of a list of numbers

        Parameters:

        *data*
            a list of numbers whose standard deviation to calculate

        """
        m = self.mean(data)
        sumsq = 0

        for d in data:
            sumsq += (d - m)**2

        return (sumsq / len(data))**(0.5)




# -*- coding: utf-8 -*-
"""
Manage Topology objects and initialize the right type based on the
configuration

If the configured Topology is not part of the standard SEEDS ones, the plugin
manager will be used to see if it has been defined by the user.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import seeds.topology
from seeds.topology import *

from seeds.PluginManager import *


class TopologyManager(object):
    """Manage the different Topology types and use the correct one

    Attributes:
        world
            A reference to the World object
        topologies
            A list of Topology objects

    """

    def __init__(self, world):
        """Initialize the TopologyManager

        Parameters:

        *world*
            A reference to the World

        """

        self.world = world
        self.topologies = []

        self.type = self.world.config.get('Experiment', 'topology')
        num_populations = self.world.config.getint('Experiment', 'populations', 1)

        for topid in xrange(num_populations):
            if self.type == 'Topology':
                t = Topology(self.world, topid)
            elif self.type == 'CartesianTopology':
                t = CartesianTopology(self.world, topid)
            elif self.type == 'MooreTopology':
                t = MooreTopology(self.world, topid)
            elif self.type == 'WellMixedTopology':
                t = WellMixedTopology(self.world, topid)
            else:
                # If the configured Topology is not one of the built-in types,
                # scan the plugins.
                if self.world.plugin_manager.plugin_exists(self.type):
                    oref = self.world.plugin_manager.get_plugin(self.type)
                    if oref == None:
                        print "Error: Couldn't find object ref for Topology type"
                    elif not issubclass(oref, Topology):
                        print "Error: Plugin %s is not an instance of Topology type" % (self.type)
                    else:
                        t = oref(self.world, topid)
                else:
                    print 'Error: Unknown Topology type'
	
        self.topologies.append(t)

    def update(self):
        """Update all topologies"""
        for top in self.topologies:
            top.update()




# -*- coding: utf-8 -*-
"""
Interface for Topologies.

A topology is a graph.  Each node contains a Cell and a ResourceManager.  If a
pair of nodes is connected, the Cells housed in those nodes are thought of as
"neighbors", and therefore can potentially interact with each other.

TODO: discuss reasoning for 1 cell per node.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman"

import random
import math

import networkx as nx

from seeds.CellManager import *
from seeds.Resource import *
from seeds.ResourceManager import *

class Topology(object):
    """
    All topologies contain properties:

        world
            Reference to the World in which it exists
        id
            Unique ID
        cell_manager
            A CellManager object to create appropriate cells.
        typeCount
            List containing the number of cells currently existing for each
            Cell type.  Updated with increment_type_count(),
            decrement_type_count(), and update_type_count() methods.
        graph
            A NetworkX graph object defining the connections between cells.

    """

    def __init__(self, world, id):
        """Initialize a Topology object.

        The topology will have no cells and an empty graph

        Parameters:

        *world*
            A reference to the World
        *id*
            A unique ID for the created Topology

        """

        self.world = world
        self.id = id
        self.typeCount = []
        self.graph = nx.Graph()
        self.cell_manager = CellManager(self.world, self)

    def __str__(self):
        """Return a string to be used when a Topology object is printed"""
        return 'Topology %d' % (self.id)

    def increment_type_count(self, type):
        """Increment the cell type count for the given type

        Parameter:

        *type*
            The cell type whose count to increment

        """
        if len(self.typeCount) <= type:
            self.typeCount.extend([0] * (1 + type-len(self.typeCount)))
        self.typeCount[type] += 1

    def decrement_type_count(self, type):
        """Decrement the cell type count for the given type

        Parameter:

        *type*
            The cell type whose count to decrement

        """

        self.typeCount[type] -= 1

    def update_type_count(self, fromtype, totype):
        """Update the cell type counts, subtracting from the 'from' type and
        adding to the 'to' type

        Parameters:

        *fromtype*
            The type that a cell was prior to being updated
        *totype*
            The type that a cell is after being updated

        """

        self.decrement_type_count(fromtype)
        x = self.typeCount[totype]
        self.increment_type_count(totype)
        if x != self.typeCount[totype]-1:
            print "ERROR!"

    def get_neighbors(self, node):
        """Get a list of neighboring cells for a given node

        Parameters:
        
        *node*
            The ID of the node whose neighboring cells to get
        
        """

        return [self.graph.node[n]['cell'] for n in self.graph.neighbors(node)]

    def size(self):
        """Get the number of nodes in the topology"""
        return len(self.graph)

    def update(self):
        """Update Cells and Resources in the Topology
        
        Update is asynchronous.  Nodes are chosen at random, and the Cell and
        Resources residing in those nodes are then updated.  The number of
        nodes to update per epoch is specified by the events_per_epoch
        parameter in the [Experiment] configuration block.  By default, the
        number of nodes to update is equal to the number of nodes in the
        Topology.
        
        """

        for x in xrange(self.world.config.getint(section='Experiment',
                                                 name='events_per_epoch',
                                                 default=len(self.graph))):
            node = random.choice(self.graph.nodes())
            self.graph.node[node]['resource_manager'].update()
            self.graph.node[node]['cell'].update(self.get_neighbors(node))




# -*- coding: utf-8 -*-
"""
Interface for Actions.

An Action is a piece of code that is executed at specified epochs and perform
things such as writing statistics, plotting graphs, or interacting with
Topologies, Cells, or Resources.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman"

import os


class Action(object):
    """
    Properties:

    world
        A reference to the World
    data_dir
        Directory in which to store created files
    epoch_start
        The epoch at which the Action starts executing
    epoch_end
        The epoch at which the Action stops executing
    frequency
        The frequency at which the Action is executed.  For example, if
        frequency=2, then the Action is executed at every other epoch.
    name
        The name of the action
    
    Configuration: The data_dir parameter should be set in the [Experiment]
    block.  Each Action should have its own configuration block.

    """

    def __init__(self, world):
        """Create an Action instance"""
        self.world = world
        self.data_dir = self.world.config.get('Experiment', 'data_dir', 'data')
        self.epoch_start = 0
        self.epoch_end = self.world.config.getint('Experiment', 'epochs', default=-1)
        self.frequency = 1
        self.name = ""

    def __str__(self):
        """Produce a string to be used when an Action object is printed"""
        return 'Action Object (%s) (epoch start: %d)(epoch end: %d)(frequency: %d)' % (self.name, self.epoch_start, self.epoch_end, self.frequency)

    def update(self):
        """ Execute the action """
        pass

    def skip_update(self):
        """ Return a boolean indicating whether or not the action should be
        executed during the current epoch

        """

        return (self.world.epoch < self.epoch_start or
                (self.epoch_end != -1 and self.world.epoch > self.epoch_end) or
                (self.world.epoch - self.epoch_start) % self.frequency != 0)

    def datafile_path(self, filename):
        """ Return the relative path to a given data file
        Parameters:

        *filename*
            The name of the file whose path to get

        """
        return os.path.join(self.data_dir, filename)




# -*- coding: utf-8 -*-
"""
Write a snapshot of the population
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import csv

from seeds.Action import *

class WriteSnapshot(Action):
    """ Write the number of cells of each type for all populations

        Config file settings:
        [WriteSnapshot]
        epoch_start = 3    Epoch at which to start writing (default 0)
        epoch_end = 100    Epoch at which to stop writing (default end of experiment)
        frequency = 2      Frequency (epochs) to write.  In this example, we write every other epoch.  (default 1)
        filename = snapshot  Base filename to be written to (will be <filename>-<epoch>.snp)

    """

    def __init__(self, world):
        """Initialize the WriteSnapshot Action"""

        Action.__init__(self, world)
        self.epoch_start = self.world.config.getint('WriteSnapshot', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('WriteSnapshot', 'epoch_end', default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('WriteSnapshot', 'frequency', 1)
        self.filename = self.world.config.get('WriteSnapshot', 'filename', 'snapshot')
        self.name = "WriteSnapshot"

    def update(self):
        """Execute the action"""
        if self.skip_update():
	        return

        data_file = self.datafile_path(self.filename)
        self.world.get_snapshot().write(data_file)


class TESTSnapshot(Action):
    """ Write the number of cells of each type for all populations

        Config file settings:
        [WriteSnapshot]
        epoch_start = 3    Epoch at which to start writing (default 0)
        epoch_end = 100    Epoch at which to stop writing (default end of experiment)
        frequency = 2      Frequency (epochs) to write.  In this example, we write every other epoch.  (default 1)
        filename = snapshot  Base filename to be written to (will be <filename>-<epoch>.snp)

    """

    def __init__(self, world):
        """Initialize the WriteSnapshot Action"""

        Action.__init__(self, world)
        self.epoch_start = self.world.config.getint('WriteSnapshot', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('WriteSnapshot', 'epoch_end', default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('WriteSnapshot', 'frequency', 1)
        self.filename = self.world.config.get('WriteSnapshot', 'filename', 'snapshot')
        self.name = "WriteSnapshot"

    def update(self):
        """Execute the action"""
        if self.skip_update():
	        return

        data_file = self.datafile_path(self.filename)
        self.world.get_snapshot().write(data_file)



# -*- coding: utf-8 -*-
"""
Manage plugins for the system

Plugins allow SEEDS to be extended to suit the needs of users without adding
to the base code and adding additional dependencies.  Examples include
additional Cells, Topologies, or Actions.  Plugins may be stored in multiple
directories.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import imp
import os
import sys

class PluginManager(object):
    """ The PluginManager object keeps track of plugins in the specified plugin
    directories and allows users to query these plugins and receive references
    to the resulting objects.

    Properties:

    world
        A reference to the World
    plugin_dirs
        A list of directories that may contain plugins
    plugins
        A module object containing attributes for each loaded plugin
        

    Configuration:
    
    The directories in which plugins can be searched for are specified by the
    plugin_dirs parameter in the [Experiment] block as a comma-separated list.

    Example:

    [Experiment]
    plugin_dirs = plugins,/opt/seedsplugins,../old_plugins

    """

    def __init__(self, world):
        """Initialize a PluginManager object

        Parameters:

        *world*
            A reference to the World
        """

        self.world = world
        self.plugin_dirs = []
        self.plugins = None

        plugindirs = self.world.config.get(section='Experiment', name='plugin_dirs', default="")
        if len(plugindirs) > 0:
            for d in plugindirs.split(','):
                if os.path.exists(d):
                    self.plugin_dirs.append(d)

        self.load_plugins()

    def load_plugins(self):
        """Scan the plugin directories and load the plugins"""

        self.plugins = None

        for plugindir in self.plugin_dirs:
            if os.path.exists(plugindir) and os.path.isdir(plugindir):
                for f in os.listdir(plugindir):
                    basename, extension = os.path.splitext(f)
                    tgt = os.path.join(plugindir, f)

                    if extension == ".py":
                        self.plugins = imp.load_source("obj", tgt)
                    elif extension == ".pyc":
                        self.plugins = imp.load_compiled("obj", tgt)

    def add_dir(self, dir):
        """Add a directory to the list of plugin directories.  After running,
        load_plugins() is called to maintain an up-to-datee list of plugins

        """

        if os.path.exists(dir):
            if self.plugin_dirs.count(dir) == 0:
                self.plugin_dirs.append(dir)
                self.load_plugins()

    def get_plugin(self, plugin=""):
        """Get a reference to the plugin.  The result may be then used to
        create new instances of that class or be executed as a function.

        Example:

        obj_ref = get_plugin("MyObject")
        new_object = obj_ref()

        After this has executed, new_object will be an object of type MyObject.

        """

        try:
            ref = getattr(self.plugins, plugin)
            return ref
        except AttributeError:
            return None

    def plugin_exists(self, plugin=""):
        """Determine if a named plugin is present in the plugin directories"""
        try:
            ref = getattr(self.plugins, plugin)
            return True
        except AttributeError:
            return False




# -*- coding: utf-8 -*-
"""
Print the coordinates of each bion cell
Note this uses the same configs as PrintCellLocations
"""

__author__ = "Michael Vo <vomichae@msu.edu>"
__credits__ = "Brian Connelly"

import csv

from seeds.Action import *

class PrintBionCellLocations(Action):
    """ Write the x,y coordinates of each cell and its color and cooperation status

    """
    def __init__(self, world):
        """Initialize the PrintBion CellLocations Action"""
        Action.__init__(self, world)

        self.epoch_start = self.world.config.getint('PrintCellLocations', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('PrintCellLocations', 'epoch_end',
                                                  default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('PrintCellLocations', 'frequency', 1)
        self.filename = self.world.config.get('PrintCellLocations', 'filename', 'cell_locations')
        self.name = "PrintBionCellLocations"

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        filename = "%s-%06d.csv" % (self.filename, self.world.epoch)
        data_file = self.datafile_path(filename)
        self.writer = csv.writer(open(data_file, 'w'))

        self.writer.writerow(['#epoch','population','cell id','x','y','type'])

        for top in self.world.topology_manager.topologies:
            for n in top.graph.nodes():
                cell = top.graph.node[n]['cell']
                row = [self.world.epoch, top.id, cell.id, cell.coords[0], cell.coords[1], cell.red, cell.green, cell.blue, cell.public_good_gene, cell.public_good_level]
                self.writer.writerow(row)




# -*- coding: utf-8 -*-
"""
Provide resources which the cells can consume or produce.

For each configured resource, a Resource object is placed into each node in the
topology. These Resource objects are updated whenever that node is updated.

Additionally, Resources can be manipulated with Actions.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

class Resource(object):
    """Environmental Resource class

    Properties:

    name
        Unique name of the resource
    level
        current level of the resource
    inflow
        amount of resource (in units) that flows into the
        environment per epoch (default: 0.0)
    outflow
        fraction of resource (in percent) that flows out of
        the environment per epoch (default: 0.0)
    initial
        the amount of resource (in units) present in the
        environment at the beginning (default: 0.0)

    Resources can be defined in config files using a [Resource:<uniquename>]
    section.  For example:

        [Resource:glucose]
        initial = 0
        inflow = 0.24
        outflow = 0.1

    At each update, the outflow amount is removed first, then inflow is added.
    This leads to equilibrium levels (if unpreturbed) of inflow/outflow

    """

    def __init__(self, name=None, inflow=0.0, outflow=0.0, initial=0.0):
        """ Initialize a Resource object

        Parameters:

        *name*
            A name for the resource
        *inflow*
            An inflow amount (in units) that flows into the cell at each update
        *outflow*
            An amount (percentage of current level) of resource that flows out
            of the cell at each update
        *initial*
            The amount of resource present in the cell at the beginning of the
            experiment

        """

        if name != None:
            self.name = name
        else:
            print "Error: Must supply Resource name"

        self.initial = initial
        self.level = self.initial * 1.0
        self.inflow = inflow
        self.outflow = outflow

        if self.outflow < 0 or self.outflow > 1:
            print "ERROR: invalid outflow"

    def __str__(self):
        """Produce a string to be used when a Resource object is printed"""
        return "Resource [Name: %s][Level: %f][Inflow: %f][Outflow: %f]" % (self.name, self.level, self.inflow, self.outflow)

    def set_name(self, value):
        """Set the name of the resource

        Parameters:

        *value*
            The new name for the resoruce

        """
        self.name = value

    def set_level(self, value):
        """Set the level of the resource

        Parameters:

        *value*
            The new level of the resoruce

        """
        self.level = value

    def set_inflow(self, value):
        """Set the inflow amount of the resource

        Parameters:

        *value*
            The new inflow amount of the resoruce

        """
        self.inflow = value

    def set_outflow(self, value):
        """Set the outflow rate of the resource

        Parameters:

        *value*
            The new outflow rate of the resoruce

        """
        self.outflow = value

    def set_initial(self, value):
        """Set the initial level of the resource

        Parameters:

        *value*
            The new initial level of the resoruce

        """
        self.initial = value

    def update(self):
        """Update the level of the resource"""
        newlevel = (self.level * (1 - self.outflow)) + self.inflow
        self.level = max(0, newlevel)




# -*- coding: utf-8 -*-
"""
Manage Action objects

If a configured Action is not part of the standard SEEDS list, the plugin
manager will be used to see if it has been defined by the user.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman"

import datetime
import os
import re
import shutil

import seeds.action
from seeds.action import *

from seeds.PluginManager import *


class ActionManager(object):
    """Manage the creation and execution of a set of Actions

    Attributes:

    actions
        The list of Action objects to be run

    """

    def __init__(self, world):
        """Initialize the ActionManager

        Parameters:

        *world*
            A reference to the World

        """

        self.world = world

        data_dir = self.world.config.get(section='Experiment', name='data_dir',
                                         default='data')

        if os.path.exists(data_dir):
            newname = data_dir + '-' + datetime.datetime.now().strftime("%Y%m%d%H%M%S")
            shutil.move(data_dir, newname)

        os.mkdir(data_dir)

        self.actions = []
        self.setup_actions()

    def setup_actions(self):
        """Initialize and set up the list of Actions to be executed"""

        actionstring = self.world.config.get(section='Experiment',
                                             name='actions', default="")

        if len(actionstring) > 0:
            actionlist = re.split('\W+', actionstring)
            for action in actionlist:
                if action == 'PrintCellLocations':
                    a = PrintCellLocations(self.world)
                elif action == 'PrintCellTypeCount':
                    a = PrintCellTypeCount(self.world)
                elif action == 'PrintGraphProperties':
                    a = PrintGraphProperties(self.world)
                elif action == 'PrintResourceStats':
                    a = PrintResourceStats(self.world)
                elif action == 'ResourceActions':
                    a = ResourceActions(self.world)
                elif action == 'StopOnConvergence':
                    a = StopOnConvergence(self.world)
                elif action == 'WriteSnapshot':
                    a = WriteSnapshot(self.world)
                else:
                    # If the configured Topology is not one of the built-in types,
                    # scan the plugins.
                    if self.world.plugin_manager.plugin_exists(action):
                        oref = self.world.plugin_manager.get_plugin(action)
                        if oref == None:
                            print "Error: Couldn't find object ref for Action type"
                        elif not issubclass(oref, Action):
                            print "Error: Plugin %s is not an instance of Action type" % (action)
                        else:
                            a = oref(self.world)
                    else:
                        print 'Error: Unknown Action type %s' % (action)

                self.actions.append(a)

    def update(self):
        """Update all actions"""
        [a.update() for a in self.actions]




# -*- coding: utf-8 -*-
from seeds.action.PrintCellLocations import *
from seeds.action.PrintCellTypeCount import *
from seeds.action.PrintGraphProperties import *
from seeds.action.PrintResourceStats import *
from seeds.action.ResourceActions import *
from seeds.action.SnapshotActions import *
from seeds.action.StopOnConvergence import *



# -*- coding: utf-8 -*-
"""
Lattice topology in which each cell is connected with each of its 8 neighbors
(Moore Neighborhood).  The radius of interactions can be defined, which means
all cells within this many hops will be considered a neighbor.

This topology was originally presented and used used in the publication:

    B.D. Connelly, L. Zaman, C. Ofria, and P.K. McKinley, "Social structure and
    the maintenance of biodiversity," in Proceedings of the 12th International
    Conference on the Synthesis and Simulation of Living Systems (ALIFE), pp.
    461-468, 2010.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman, Philip McKinley, Charles Ofria"

import random
import math

from seeds.CellManager import *
from seeds.topology.Topology import *
from seeds.Resource import *
from seeds.ResourceManager import *

import networkx as nx

class MooreTopology(Topology):
    """
    Lattice topology with Moore Neighborhoods with configurable radius

    Cells are organized on a lattice.  A cell's neighbors reside to the
    left, right, above, below, and on the diagonals.  With radius 1, this
    neighborhood will contain 8 cells.  With radius 2, the 24 cells within
    a 2-hop distance are included.

    Configuration: All configuration options should be specified in the
        MooreTopology block.

        size: Width/height, in number of cells, of the world.  With size
            10, there would be 100 cells. (no default)
        periodic_boundaries: Whether or not the world should form a torus.
            This means that cells on the left border are neighbors with
            cells on the right border. (default: False)
        radius: Number of hops within a focal cell's neighborhood
            (default: 1)

    Example:
        [MooreTopology]
        size = 100
        periodic_boundaries = True
        radius = 4


    """
    def __init__(self, world, id):
        """Initialize a MooreTopology object"""
        Topology.__init__(self, world, id)

        self.size = self.world.config.getint('MooreTopology', 'size')
        self.periodic_boundaries = self.world.config.getboolean('MooreTopology', 'periodic_boundaries', default=False)
        self.radius = self.world.config.getint('MooreTopology', 'radius', default=1)

        if self.radius >= self.size:
            print 'Error: Radius cannot be bigger than world!'

        self.graph = self.moore_2d_graph(self.size, self.size,
                                         radius=self.radius,
                                         periodic_boundaries=self.periodic_boundaries)

        for n in self.graph.nodes():
            self.graph.node[n]['cell'] = self.cell_manager.newcell(n, n)
            self.graph.node[n]['cell'].coords = (self.row(n), self.column(n))
            self.graph.node[n]['resource_manager'] = ResourceManager(world, self)

    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return 'Moore Topology (%d cells, %d radius)' % (self.size * self.size, self.radius)

    def row(self, cellid):
        """Get the number of the row in which the given cell is located

        Parameters:

        *cellid*
            The ID of the cell in question

        """
        return (cellid/self.size)

    def column(self, cellid):
        """Get the number of the column in which the given cell is located

        Parameters:

        *cellid*
            The ID of the cell in question

        """
        return cellid % self.size

    def cell_id(self, row, col):
        """Get the ID of the cell at the given row and column

        Parameters:

        *row*
            The row in which the cell is located
        *col*
            The column in which the cell is located

        """
        return row * self.size + col

    def moore_2d_graph(self, rows=0, columns=0, radius=0,
                       periodic_boundaries=False):
        """ Return the 2d grid graph of rows x columns nodes,
            each connected to its nearest Moore neighbors within
            a given radius.
            Optional argument periodic=True will connect
            boundary nodes via periodic boundary conditions.

        Parameters:

        *rows*
            The number of rows to be in the graph
        *columns*
            The number of columns to be in the graph
        *radius*
            The radius of interactions in the graph (there will be an edge
            between a cell and all other cells within N hops)
        *periodic_boundaries*
            Prevent edge effects using periodic boundaries

        """
        G = nx.empty_graph()
        G.name = "moore_2d_radius_graph"
        G.add_nodes_from(range(rows * columns))

        for n in G.nodes():
            myrow = self.row(n)
            mycol = self.column(n)

            for r in range(myrow - radius, myrow + radius + 1):
                if periodic_boundaries == False and (r < 0 or r >= rows):
                    continue

                for c in range(mycol - radius, mycol + radius + 1):
                    if periodic_boundaries == False and (c < 0 or c >= columns):
                        continue

                    cid = self.cell_id(r % rows, c % columns)

                    if cid != n:
                        neighbor_id = ((r % rows) * self.size) + (c % columns)
                        G.add_edge(n, neighbor_id)

        return G




# -*- coding: utf-8 -*-
"""
Moves museum visitors to new locations and prints them, and chooses what cell gets an energy boost
"""

__author__ = "Michael Vo <vomichae@msu.edu>"
__credits__ = "Brian Connelly"

import csv
import random

from seeds.Action import *

class Visitor(object):
    def __init__(self, world, location):
        self.world = world
        self.location = location

        self.pref_red = self.world.config.getint('Bion', 'red', 0)
        self.pref_green = self.world.config.getint('Bion', 'green', 0)
        self.pref_blue = self.world.config.getint('Bion', 'blue', 0)

        self.time_at_location = 0
        self.current_score = 9999999

        self.visited_sites = []

class MovePerson(Action):
    """ Write the x,y coordinates of each visitor and moves them
    Also gives fitness benefits to cells with high similarity to the color they seek
    Config file settings are in Bion
    """
    def __init__(self, world):
        """Initialize the MovePerson Action"""
        Action.__init__(self, world)
        self.top = self.world.topology_manager.topologies[0]
       	self.name = "MovePerson"

       	self.red = self.world.config.getint('Bion', 'red', 0)
       	self.green=self.world.config.getint('Bion', 'green', 0)
       	self.blue=self.world.config.getint('Bion', 'blue', 1)
       	self.num_people = self.world.config.getint('Bion', 'people', 1)
       	self.neighbor_visit_frac = self.world.config.getfloat('Bion', 'neighbor_visit_frac', 0.25)

        self.visitors = []
        for i in xrange(self.num_people):
            node = random.choice(self.top.graph.nodes())
            cell = self.top.graph.node[node]['cell']
            v = Visitor(world, cell)
            self.visitors.append(v)

    def update(self):
        filename = "%s-%06d.csv" % ("person", self.world.epoch)
        data_file = self.datafile_path(filename)
        self.writer = csv.writer(open(data_file, 'w'))
        self.writer.writerow(['#epoch', 'population', 'cell id','x','y','type'])
    	
    	"""
    	The code below finds the neighbor with the highest similarity to the target color, and gives it an energy boost.
    		
    	It also moves to a random neighbor, and writes to file
    	"""
    	for visitor in self.visitors:
            moved = False
            visitor.time_at_location += 1

            # Visit the current cell.
            visitor.location.visit(frac=1)

            neighbors = self.top.get_neighbors(visitor.location.node)
            for n in neighbors:
                visitor.location.visit(frac=self.neighbor_visit_frac)
                score = (visitor.pref_red - n.red)**2 + (visitor.pref_blue - n.blue)**2 + (visitor.pref_green - n.green)**2
                
                # Move to cell with closest color
                if score < visitor.current_score and n.id not in visitor.visited_sites:
                    current_score = score
                    visitor.location = n
                    visitor.current_score = score
                    visitor.visited_sites.append(n.id)
                    visitor.time_at_location = 0

            # If no movement has been made in N epochs, move to a random neighbor
            if not moved and visitor.time_at_location > 3:
                visitor.location = random.choice(neighbors)
                visitor.visited_sites.append(visitor.location.id)
                visitor.time_at_location = 0
                visitor.current_score = (visitor.pref_red - visitor.location.red)**2 + (visitor.pref_blue - visitor.location.blue)**2 + (visitor.pref_green - visitor.location.green)**2

            self.writer.writerow([self.world.epoch, self.top.id, visitor.location.id, visitor.location.coords[0], visitor.location.coords[1]])




# -*- coding: utf-8 -*-
"""
Suite of Actions to change properties of a given Resource in all
populations
"""
__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import csv

from seeds.Action import *

class AdjustResource(Action):

    """ Adjust a given Resource.  This could change the current level, the
    inflow amount, or the outflow rate.


    Configuration: All configuration options should be specified in a
    AdjustResource block.

    epoch_start
        The epoch at which the Action starts executing
    epoch_end
        The epoch at which the Action stops executing
    frequency
        The frequency at which the Action is executed.  For example, if
        frequency=2, then the Action is executed at every other epoch.
    resource
        The name of the resource to adjust
    type
        The action to take on that resource.  One of:

        add - Add <value> amount to current resource level
        remove - Remove <value> from current resource level
        clear - Remove all resource
        set - Set the current level of resource to <value>
        setinflow - Set the inflow of resource to <value>
        setoutflow - Set the outflow of resource to <value>
    value
        Value corresponding to the action

    Example:

    [AdjustResource]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    resource = glucose
    type = add
    value = 1.2

    """
    def __init__(self, world):
        """Initialize the AdjustResource object based on values in
        [AdjustResource] config block
        
        """

        Action.__init__(self, world)

        self.epoch_start = self.world.config.getint('AdjustResource', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('AdjustResource', 'epoch_end', default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('AdjustResource', 'frequency', 1)

        self.resource = self.world.config.get('AdjustResource', 'resource')
        self.type = self.world.config.get('AdjustResource', 'type')
        self.value = self.world.config.getfloat('AdjustResource', 'value', 0.0)

        if (self.type != 'add' and self.type != 'remove' and
            self.type != 'clear' and self.type != 'set' and
            self.type != 'setinflow' and self.type != 'setoutflow'):
            print 'Error: Invalid type for AdjustResource'
      
    def update(self):
        """Adjust the appropriate resource accordingly"""
        if self.skip_update():
	        return

        for top in self.world.topology_manager.topologies:
            for cell in top.cells:
                if self.type == 'add':
                    res = cell.resource_manager.get_resource(self.resource)
                    if res != None:
                        res.set_level(res.level + self.value)
                elif self.type == 'remove':
                    res = cell.resource_manager.get_resource(self.resource)
                    if res != None:
                        res.set_level(res.level - self.value)
                elif self.type == 'clear':
                    res = cell.resource_manager.get_resource(self.resource)
                    if res != None:
                        res.level = 0
                        res.set_level(0)
                elif self.type == 'set':
                    res = cell.resource_manager.get_resource(self.resource)
                    if res != None:
                        res.set_level(self.value)
                elif self.type == 'setinflow':
                    res = cell.resource_manager.get_resource(self.resource)
                    if res != None:
                        res.set_inflow(self.value)
                elif self.type == 'setoutflow':
                    res = cell.resource_manager.get_resource(self.resource)
                    if res != None:
                        res.set_outflow(self.value)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Script to perform a SEEDS experiment.  For more information, run with the
--help argument.  Plugin directories can be specified in the configuration as
well as in the $SEEDSPLUGINPATH environment variable, which allows for a user-
or system-wide repository of plugins.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__version__ = "1.0.4"
__credits__ = "Brian Connelly"

import seeds as S
from optparse import OptionParser

import os
import re
import sys

class ProgressBar:
    def __init__(self, min_value = 0, max_value = 0, width=60):
        self.bar = ''
        self.min = min_value
        self.max = max_value
        self.width = width
        self.amount = 0       # When amount == max, we are 100% done
        self.update_amount(0)

    def update_amount(self, new_amount = None):
        """
        Update self.amount with 'new_amount', and then rebuild the bar
        string.
        """
        if not new_amount: new_amount = self.amount
        if new_amount < self.min: new_amount = self.min
        if new_amount > self.max: new_amount = self.max
        self.amount = new_amount
        self.build_bar()

    def build_bar(self):
        """
        Figure new percent complete, and rebuild the bar string base on
        self.amount.
        """
        diff = float(self.amount - self.min)
        span = self.max - self.min
        percent_done = int(round((diff / float(span)) * 100.0))

        # figure the proper number of 'character' make up the bar
        all_full = self.width - 2
        num_hashes = int(round((percent_done * all_full) / 100))

        # build a progress bar with self.char and spaces (to create a
        # fixed bar (the percent string doesn't move)
        percent_str = "epoch: " + str(self.amount)
        self.bar = '#' * num_hashes + ' ' * (all_full-num_hashes)
        self.bar = '[' + self.bar + '] ' + percent_str

    def __str__(self):
        return str(self.bar)


def main():

    parser = OptionParser('usage: %prog [options] arg')
    parser.add_option("-c", "--config", dest="configfile", type="string", default="seeds.cfg",
                      help="read config file (default: seeds.cfg)")
    parser.add_option("-C", "--genconfig", action="store_true", dest="genconfig", help="write config file used (experiment.cfg)")
    parser.add_option("-d", "--data_dir", dest="datadir", type="string", default="data",
                      help="write data to this directory (default: data)")
    parser.add_option("-p", "--param", dest="params", type="string", help="Set config values.  Semicolon-separated list of section.param=val")
    parser.add_option("-q", "--quiet", action="store_true", dest="quiet", help="suppress all output messages")
    parser.add_option("-r", "--resume", dest="snapfile", type="string", help="resume experiment from the provided snapshot file")
    parser.add_option("-s", "--seed", dest="seed", type=int, default=0,
                      help="set random seed (default: use clock)")
    parser.add_option("-S", "--snapshot", action="store_true", dest="snapshot", help="write snapshot file at end of experiment")
    parser.add_option("-v", "--version", action="store_true", dest="version", help="display version information and quit")

    (cmd_options, cmd_args) = parser.parse_args()

    if cmd_options.seed > 0:
        random_seed = cmd_options.seed
    else:
        random_seed=-1

    if cmd_options.version:
        print "%s (SEEDS Version %s)" % (__version__, S.__version__)
        sys.exit(0)

    # Create the world...
    # TODO: what if data_dir is set in config file?
    world = S.World(configfile=cmd_options.configfile, seed=random_seed)
    world.config.set('Experiment', 'data_dir', cmd_options.datadir)


    # Load the state of the world from a snapshot
    if cmd_options.snapfile:
        if not os.path.exists(cmd_options.snapfile):
            print "Error: Snapshot file does not exist"
            sys.exit(1)
        world.load_snapshot(cmd_options.snapfile)


    # Add command-line config options
    if cmd_options.params != None:
        options = re.split("\s*;\s*", cmd_options.params)
        for opt in options:
            # This is perhaps not the best regexp for comma-separated lists as values... need spaces.
            m = re.match(r"(?P<section>[A-Za-z0-9:_]+)\.(?P<parameter>[A-Za-z0-9:_]+)\s*=\s*(?P<value>-?[A-Za-z0-9_\.\,]+)", opt)
            if m != None:
                #print "Setting [%s]%s=%s" % (m.group("section"), m.group("parameter"), m.group("value"))
                world.config.set(m.group("section"), m.group("parameter"), m.group("value"))
            else:
                print "Error: Could not parse parameter setting", opt


    # Get the current configured list of plugin directories
    cfg_plugindirs = world.config.get(section="Experiment", name="plugin_dirs",
                                      default="")
    if len(cfg_plugindirs) > 0:
        plugindirs = world.config.get(section="Experiment", name="plugin_dirs").split(",")
    else:
        plugindirs = []

    # Add any plugin paths specified in the environment via $SEEDSPLUGINPATH
    seedspluginpath = os.environ.get("SEEDSPLUGINPATH")
    if seedspluginpath != None and len(seedspluginpath) > 0:
        for p in seedspluginpath.rsplit(":"):
            pdir = os.path.expanduser(p)
            plugindirs.append(pdir)

    if len(plugindirs) > 0:
        pdirs = ",".join(plugindirs)
        world.config.set(section="Experiment", name="plugin_dirs", value=pdirs)

    # Do the experiment...
    prog = ProgressBar(0, world.config.getint('Experiment', 'epochs'))
    oldprog = str(prog)

    while world.proceed:
        prog.update_amount(world.epoch)
        if not cmd_options.quiet and oldprog != str(prog):
            print prog, "\r",
            sys.stdout.flush()
            oldprog=str(prog)

        world.update()

    # Write a config file
    if cmd_options.genconfig:
        world.config.write(filename='experiment.cfg')

    # Write a snapshot at the end
    if cmd_options.snapshot:
        snap = world.get_snapshot()
        snap.write()

    if not cmd_options.quiet:
        print

if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
"""
Print a number of measures related to the graphs used for topologies

Note that this action may take considerable time to execute for large
topologies.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import networkx as nx

import csv

from seeds.Action import *

class PrintGraphProperties(Action):
    """ Write various properties of the graphs used

        Config file settings:
        [PrintGraphProperties]
        epoch_start = 3    Epoch at which to start writing (default 0)
        epoch_end = 100    Epoch at which to stop writing (default end of experiment)
        frequency = 2      Frequency (epochs) to write.  In this example, we write every other epoch.  (default 1)
        filename = graph_properties.dat  Filename to be written to

    """
    def __init__(self, world):
        """Initialize the PrintGraphProperties Action"""
        Action.__init__(self, world)

        self.epoch_start = self.world.config.getint('PrintGraphProperties', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('PrintGraphProperties', 'epoch_end', default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('PrintGraphProperties', 'frequency', 1)
        self.filename = self.world.config.get('PrintGraphProperties', 'filename', 'graph_properties.dat')
        self.name = "PrintGraphProperties"

        data_file = self.datafile_path(self.filename)
        self.writer = csv.writer(open(data_file, 'w'))

        self.writer.writerow(['#epoch','population','nodes','edges','avg. degree', 'std. degree', 'avg. clustering coefficient','diameter', 'num connected components'])
      
    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        for top in self.world.topology_manager.topologies:
            degrees = nx.degree(top.graph).values()
            row = [self.world.epoch, top.id, nx.number_of_nodes(top.graph), nx.number_of_edges(top.graph), self.mean(degrees), self.std(degrees), nx.average_clustering(top.graph), nx.diameter(top.graph), nx.number_connected_components(top.graph)]
            self.writer.writerow(row)

    def mean(self, data):
        """Calculate the mean of a list of numbers

        Parameters:

        *data*
            a list of numbers whose mean to calculate

        """
        return float(sum(data))/len(data)

    def std(self, data):
        """Calculate the standard deviation of a list of numbers

        Parameters:

        *data*
            a list of numbers whose standard deviation to calculate

        """
        m = self.mean(data)
        sumsq = 0

        for d in data:
            sumsq += (d - m)**2

        return (sumsq / len(data))**(0.5)




# -*- coding: utf-8 -*-
"""
Handle a collection of Resource objects in a Cell
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import re

from seeds.Resource import *

class ResourceManager(object):
    """
    Manage a Cell's Resources

    Each Cell object has a ResourceManager, which maintains a list of the
    Resources in that cell.

    """

    def __init__(self, world, topology):
        """Initialize a ResourceManager object

        Parameters:
        
        *world*
            A reference to the World
        *topology*
            The topology in which the Cell and Resources are located

        """
        self.world = world
        self.topology = topology
        self.resources = []
        self.init_resources()

    def init_resources(self):
        """Initialize all resources that will be associated with this Cell"""
        for res in self.world.config.get_resource_sections():
            match = re.match("Resource:(?P<resname>[a-zA-Z_]+)", res)
            if match != None:
                name = match.group("resname")

                initial = self.world.config.getfloat(res, 'initial')
                inflow = self.world.config.getfloat(res, 'inflow')
                outflow = self.world.config.getfloat(res, 'outflow')

                r = Resource(name=name, initial=initial, inflow=inflow, outflow=outflow)
                self.add_resource(r)

    def add_resource(self, newres):
        """Add a resource"""
        self.resources.append(newres)

    def get_level(self, name):
        """Get the level of a given resource

        Parameters:

        *name*
            The name of the Resource in question

        """
        for res in self.resources:
            if res.name == name:
                return res.level

    def get_resource(self, name):
        """Get the object associated with a given resource

        Parameters:

        *name*
            The name of the Resource in question

        """
        for res in self.resources:
            if res.name == name:
                return res

    def update(self):
        """Update all of the managed Resource objects"""
        for res in self.resources:
            res.update()




# -*- coding: utf-8 -*-
"""
Manage Cell objects and initialize the right type based on the
configuration

If the configured Cell is not part of the standard SEEDS ones, the plugin
manager will be used to see if it has been defined by the user.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman"

import seeds.cell
from seeds.cell import *

from seeds.PluginManager import *


class CellManager(object):
    """ The purpose of the cell manager is to initialize Cell objects of the
    correct type, as specified by the "cell" parameter in the [Experiment]
    block in the configuration.

    """

    def __init__(self, world, topology):
        """Initialize a CellManager object

        Parameters:

        *world*
            A reference to the World object
        *topology*
            A reference to the Topology object in which the Cell will reside

        """

        self.world = world
        self.topology = topology
        self.type = self.world.config.get('Experiment', 'cell')

    def newcell(self, node, id):
        """Create a new Cell object"""
        if self.type == 'Cell':
            c = Cell(self.world, self.topology, node, id)
        elif self.type == 'Kerr07Cell':
            c = Kerr07Cell(self.world, self.topology, node, id)
        elif self.type == 'RPSCell':
            c = RPSCell(self.world, self.topology, node, id)
        else:
            # If the configured Cell is not one of the built-in types,
            # scan the plugins.
            if self.world.plugin_manager.plugin_exists(self.type):
                oref = self.world.plugin_manager.get_plugin(self.type)
                if oref == None:
                    print "Error: Couldn't find object ref for Cell type"
                elif not issubclass(oref, Cell):
                    print "Error: Plugin %s is not an instance of Cell type" % (self.type)
                else:
                    c = oref(self.world, self.topology, node, id)
            else:
                print 'Error: Unknown Cell type'

        return c




# -*- coding: utf-8 -*-
"""
A Cell object represents a place in the world where an organism could reside.
If that Cell is occupied by an organism, the Cell object also defines that
organism.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


class Cell(object):
    """
    Interface for Cell objects

    Properties:
      world
        A reference to the World in which the Cell exists
      topology
        A reference to the specific Topology in which the Cell exists
      node
        A reference to the node on which the Cell exists (for resource access)
      id
        A unique ID representing that Cell (at minimum, unique to its Topology)
      types
        List of strings describing the possible types the Cell could be
      type
        Number indicating which type the current Cell is.  This number
        is also an index into the 'types' parameter.
      coords
        Tuple of coordinates representing the location of the Cell in the
        Topology

    Configuration:
        Configuration options for each custom Cell object should be stored in a
        configuration block bearing the name of that Cell type (e.g.,
        "[DemoCell]")

    """

    def __init__(self, world, topology, node, id):
        """Initialize a Cell object

        Parameters:
        
        *world*
            A reference to the World
        *topology*
            A reference to the Topology in which this Cell exists
        *node*
            A reference to the graph node on which the Cell exists
        *id*
            A unique ID for this cell

        """

        self.world = world
        self.topology = topology
        self.node = node
        self.id = id

    def __str__(self):
        """Produce a string to be used when a Cell object is printed"""
        return 'Cell %d Type %d' % (self.id, self.type)


    def get_neighbors(self):
        """Get a list of neighboring cells"""

    def update(self, neighbors):
        """Update the Cell given a list of neighboring Cells
        
        Parameters:

        *neighbors*
            A list of Cell objects that are neighboring this cell in the topology

        """
        pass




# -*- coding: utf-8 -*-
""" TODO: comments
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.cell.Cell import *
import random

class BDCPunishment(Cell):
    """
    TODO
    """

    types = ['Empty', 'Cooperator', 'Cooperator-Punisher', 'Noncooperator', 'Noncooperator-Punisher']

    EMPTY = 0
    COOPERATOR = 1
    COOPERATOR_PUNISHER= 2
    NONCOOPERATOR = 3
    NONCOOPERATOR_PUNISHER= 4

    def __init__(self, world, topology, node, id, type=-1):
        """Initialize a BDCPunishment object

        The type for the cell is selected at random.

        Parameters:

        *world*
            A reference to the World
        *topology*
            A reference to the topology in which the Cell will reside
        *node*
            A reference to the node on which the Cell exists
        *id*
            A unique ID for the cell
        *type*
            The type of cell to initialize (-1 for random)

        """

        Cell.__init__(self,world,topology,node,id)

        if type == -1:
            self.type = random.randint(0,len(self.types)-1)
        else:
            self.type = type
        
        self.topology.increment_type_count(self.type)

        self.death_base = self.world.config.getfloat('BDCPunishment', 'death_base')
        self.cost_coop = self.world.config.getfloat('BDCPunishment', 'cost_coop')
        self.benefit_coop = self.world.config.getfloat('BDCPunishment', 'benefit_coop')
        self.cost_punishment = self.world.config.getfloat('BDCPunishment', 'cost_punishment')
        self.punishment = self.world.config.getfloat('BDCPunishment', 'punishment')
        self.mutation = self.world.config.getfloat('BDCPunishment', 'mutation')

    def __str__(self):
        """Produce a string to be used when the object is printed"""
        return 'BDCPunishment Cell %d Type %d (%s)' % (self.id, self.type, self.types[self.type])

    def type(self):
        """Return the name of the type of this cell"""
        return self.types[self.type]

    def update(self, neighbors):
        """Update the cell based on its neighbors

        TODO: doc

        Parameters:

        *neighbors*
            A list of neighboring cells

        """

        neighbor = random.choice(neighbors)

        if self.type == self.EMPTY:
            newtype = neighbor.type

            # Loss/gain of cooperation through mutation during replication
            if newtype == self.COOPERATOR and random.random() < self.mutation:
                newtype = self.NONCOOPERATOR
            if newtype == self.COOPERATOR_PUNISHER and random.random() < self.mutation:
                newtype = self.NONCOOPERATOR_PUNISHER
            if newtype == self.NONCOOPERATOR and random.random() < self.mutation:
                newtype = self.COOPERATOR
            if newtype == self.NONCOOPERATOR_PUNISHER and random.random() < self.mutation:
                newtype = self.COOPERATOR_PUNISHER

            # Loss/gain of punishment through mutation during replication
            if newtype == self.COOPERATOR_PUNISHER and random.random() < self.mutation:
                newtype = self.COOPERATOR
            if newtype == self.NONCOOPERATOR_PUNISHER and random.random() < self.mutation:
                newtype = self.NONCOOPERATOR
            if newtype == self.COOPERATOR and random.random() < self.mutation:
                newtype = self.COOPERATOR_PUNISHER
            if newtype == self.NONCOOPERATOR and random.random() < self.mutation:
                newtype = self.NONCOOPERATOR_PUNISHER

            self.type = newtype
            self.topology.update_type_count(self.EMPTY, self.type)

        elif self.type == self.COOPERATOR:
            death = self.death_base + self.cost_coop

            if neighbor.type == self.COOPERATOR or neighbor.type == self.COOPERATOR_PUNISHER:
                death = death - self.benefit_coop

            if random.random() < death:
                self.topology.update_type_count(self.type, self.EMPTY)            
                self.type = self.EMPTY

        elif self.type == self.COOPERATOR_PUNISHER:
            death = self.death_base + self.cost_coop + self.cost_punishment

            if neighbor.type == self.COOPERATOR or neighbor.type == self.COOPERATOR_PUNISHER:
                death = death - self.benefit_coop

            if random.random() < death:
                self.topology.update_type_count(self.type, self.EMPTY)            
                self.type = self.EMPTY

        elif self.type == self.NONCOOPERATOR:
            death = self.death_base

            if neighbor.type == self.COOPERATOR_PUNISHER or neighbor.type == self.NONCOOPERATOR_PUNISHER:
                death = death + self.punishment

            if random.random() < death:
                self.topology.update_type_count(self.type, self.EMPTY)            
                self.type = self.EMPTY

        elif self.type == self.NONCOOPERATOR_PUNISHER:
            death = self.death_base + self.cost_punishment

            if neighbor.type == self.COOPERATOR_PUNISHER or neighbor.type == self.NONCOOPERATOR_PUNISHER:
                death = death + self.punishment

            if random.random() < death:
                self.topology.update_type_count(self.type, self.EMPTY)            
                self.type = self.EMPTY




# -*- coding: utf-8 -*-
""" Cell type representing the classic game Rock-Paper-Scissors (RPS).  In this
system, a cell can be either a rock, paper, or scissors.  Rock is defeated by
paper, paper is defeated by scissors, and scissors is defeated by rock.  This
is a non-transitive system, where each strategy beats one but is beaten by
another.  If any one strategy is lost, another strategy will also be lost, as
the non-transitivity is broken (e.g., if no one can play rock, scissors will
always win the game).

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.cell.Cell import *
import random

class RPSCell(Cell):
    """
    This is a simple Cell type modeling the classic Rock-Paper-Scissors (RPS)
    game.

    This Cell type has 3 states:
        ROCK: Outcompetes SCISSORS cells
        PAPER: Outcompetes ROCK cells
        SCISSORS: Outcompetes PAPER cells

    Configuration: There are no parameters configurable for this Cell type.

    """

    types = ['Rock', 'Paper', 'Scissors']

    ROCK = 0
    PAPER = 1
    SCISSORS = 2

    def __init__(self, world, topology, node, id, type=-1):
        """Initialize a RPSCell object

        The type for the cell is selected at random.

        Parameters:

        *world*
            A reference to the World
        *topology*
            A reference to the topology in which the Cell will reside
        *node*
            A reference to the node on which the Cell resides
        *id*
            A unique ID for the cell
        *type*
            The type of cell to initialize (-1 for random)

        """

        Cell.__init__(self,world,topology,node,id)

        if type == -1:
            self.type = random.randint(0,len(self.types)-1)
        else:
            self.type = type
        
        self.topology.increment_type_count(self.type)

    def __str__(self):
        """Produce a string to be used when the object is printed"""
        return 'RPSCell %d Type %d (%s)' % (self.id, self.type, self.types[self.type])

    def type(self):
        """Return the name of the type of this cell"""
        return self.types[self.type]

    def update(self, neighbors):
        """Update the cell based on a competition with a randomly-selected
        neighbor

        Rock cells will be replaced by paper cells.  Paper cells will be
        replaced by Scissors cells.  Scissors cells will be replaced by Rock
        cells.

        Parameters:

        *neighbors*
            A list of neighboring cells

        """

        # Pick a random neighbor to compete with.  If that neighbor wins, it
        # gets the current cell.
        competitor = random.choice(neighbors)

        if self.type == self.ROCK and competitor.type == self.PAPER:
            self.type = self.PAPER
            self.topology.update_type_count(self.ROCK, self.type)            
        elif self.type == self.PAPER and competitor.type == self.SCISSORS:
            self.type = self.SCISSORS
            self.topology.update_type_count(self.PAPER, self.type)            
        elif self.type == self.SCISSORS and competitor.type == self.ROCK:
            self.type = self.ROCK
            self.topology.update_type_count(self.SCISSORS, self.type)            




# -*- coding: utf-8 -*-
"""
Topology in which the location of each Cell is represented by points randomly
placed on a 2D plane.  Each Cell is connected to a configured expected number
of neighbors.

This topology was originally presented and used used in the publication:

    B.D. Connelly, L. Zaman, C. Ofria, and P.K. McKinley, "Social structure and
    the maintenance of biodiversity," in Proceedings of the 12th International
    Conference on the Synthesis and Simulation of Living Systems (ALIFE), pp.
    461-468, 2010.

"""

__author__ = "Luis Zaman <zamanlui@msu.edu>"
__credits__ = "Luis Zaman, Brian Connelly, Philip McKinley, Charles Ofria"

import random
import math
from math import sqrt, floor, ceil, pi

from seeds.CellManager import *
from seeds.topology.Topology import *


class CartesianTopology(Topology):
    """
    Topology based on points in Cartesian space on a 2D plane

    Points are placed randomly on a unit two-dimensional Cartesian plane.
    Each cell is connected to those cells that fall within a given distance.
    This distance is calculated to yield an expected number of neighbors
    equal to the expected_neighbors configuration parameter.

    Configuration: All configuration parameters are specified in a
        [CartesianTopology] block.

    size
        Total number of Cells in the topology (Integer)
    periodic_boundaries
        Whether or not to use periodic boundary conditions, which connects the
        edges of the plane, forming a torus. (Boolean)
    expected_neighbors
        The number of neighbors (expected) each cell will have.
    remove_disconnected
        Whether or not to remove nodes that do not have neighbors within the
        calculated radius.  If False, node is connected to a randomly-chosen
        neighbor (Boolean.  Default: True)

    Example:

    [CartesianTopology]
    size = 100000
    periodic_boundaries = True
    expected_neighbors = 20

    """

    def __init__(self, world, id):
        """Initialize a CartesianTopology object

        Parameters:

        *world*
            A reference to the World
        *id*
            A unique ID assigned to the created CartesianTopology

        """

        Topology.__init__(self, world, id)
        self.size = self.world.config.getint('CartesianTopology', 'size')
        self.periodic_boundaries = self.world.config.getboolean('CartesianTopology', 'periodic_boundaries', default=False)
        self.expected_neighbors = self.world.config.getint('CartesianTopology', 'expected_neighbors')
        self.remove_disconnected = self.world.config.getboolean('CartesianTopology', 'remove_disconnected', default=True)

        self.graph = self.build_graph(size=self.size,
                                      expected_neighbors=self.expected_neighbors,
                                      periodic_boundaries=self.periodic_boundaries)

        # TODO: what is 120?
        #if self.expected_neighbors != -1 and self.expected_neighbors > 1 and self.expected_neighbors < 120:
        #    self.radius = math.sqrt( (self.expected_neighbors / float(self.size - 1 )) / math.pi)


    def build_graph(self, size=0, expected_neighbors=0,
                    periodic_boundaries=False):
        """Build the graph

        Parameters:

        *size*
            The number of nodes to be in the graph
        *expected_neighbors*
            The expected degree of each node in the graph
        *periodic_boundaries*
            Whether or not to use periodic boundary conditions

        """

        # Calculate the distance required to yield the expected # neighbors
        radius = sqrt( (expected_neighbors / (size - 1.0)) / pi)

        # Create bins in which to put cells so we only check a fraction of
        # candidate neighbors
        num_bins = int(ceil(1/radius))
        neighbor_bins = []
        for i in xrange(num_bins):
            neighbor_bins.append([])
            for j in xrange(num_bins):
                neighbor_bins[i].append([])

        G = nx.empty_graph()
        G.name = "Cartesian Topology Graph"
        G.add_nodes_from(range(size))


        # Create the collection of nodes and put them into bins with
        # candidate neighbors
        for n in G.nodes():
            G.node[n]['resource_manager'] = ResourceManager(self.world, self)
            G.node[n]['cell'] = self.cell_manager.newcell(n,n)
        
            xcoord = random.random()
            ycoord = random.random()
            G.node[n]['cell'].coords = (xcoord, ycoord)

            # Put node into bin with candidate neighbors
            bin_x = int(floor(xcoord/radius))
            bin_y = int(floor(ycoord/radius))
            neighbor_bins[bin_x][bin_y].append(n)


        # Find actual neighbors and create edges between nodes
        for x in xrange(num_bins):
            for y in xrange(num_bins):

                # Get all potential neighbors (those in adjacent bins)
                potentials = []
                for px in range(x-1, x+1+1):
                    if (periodic_boundaries == False and
                        (px < 0 or px >= num_bins)):
                        continue

                    for py in range(y-y, y+1+1):
                        if (periodic_boundaries == False and
                            (py < 0 or py >= num_bins)):
                            continue

                        potentials += neighbor_bins[px % num_bins][py % num_bins]

                for node in neighbor_bins[x][y]:
                    node_coords = G.node[node]['cell'].coords
                    for potential in potentials:
                        p_coords = G.node[potential]['cell'].coords
                        if (self.within_range(node_coords, p_coords, radius,
                                              periodic_boundaries) and
                            node != potential):
                            G.add_edge(node, potential)

                    if G.degree(node) == 0:
                        if self.remove_disconnected:
                            G.remove_node(node)
                            neighbor_bins[x][y].remove(node)
                        else:
                            G.add_edge(node, random.choice(potentials))

                neighbor_bins[x][y] = []

        return G


    def distance(self, node1, node2, periodic_boundaries):
        """Calculate the distance between two nodes

        Parameters:

        *node1*
            The first node
        *node2*
            The second node
        *periodic_boundaries*
            Whether or not periodic boundary conditions are used.

        """

        if periodic_boundaries:
            dx = abs(node1[0] - node2[0])
            dist_x = min(dx, 1-dx)
            dy = abs(node1[1] - node2[1])
            dist_y = min(dy, 1-dy)
        else:
            dist_x = node1[0] - node2[0]
            dist_y = node1[1] - node2[1]

        return sqrt(dist_x**2 + dist_y**2)


    def within_range(self, node1, node2, distance, periodic_boundaries):
        """Determine whether or not two nodes are within a given distance from
        each other

        Parameters:

        *node1*
            The first node
        *node2*
            The second node
        *distance*
            The threshold distance 
        *periodic_boundaries*
            Whether or not periodic boundary conditions are used.

        """

        return self.distance(node1, node2, periodic_boundaries) < distance




# -*- coding: utf-8 -*-
"""
Print the number of Cells for each cell type for all populations
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import csv

from seeds.Action import *

class PrintCellTypeCount(Action):
    """ Write the number of cells of each type for all populations

        Config file settings:
        [PrintCellTypeCount]
        epoch_start = 3    Epoch at which to start writing (default 0)
        epoch_end = 100    Epoch at which to stop writing (default end of experiment)
        frequency = 2      Frequency (epochs) to write.  In this example, we write every other epoch.  (default 1)
        filename = cell_type_count.dat  Filename to be written to

    """

    def __init__(self, world):
        """Initialize the PrintCellTypeCount Action"""

        Action.__init__(self, world)
        self.epoch_start = self.world.config.getint('PrintCellTypeCount', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('PrintCellTypeCount', 'epoch_end', default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('PrintCellTypeCount', 'frequency', 1)
        self.filename = self.world.config.get('PrintCellTypeCount', 'filename', 'cell_type_count.dat')
        self.name = "PrintCellTypeCount"

        data_file = self.datafile_path(self.filename)
        self.writer = csv.writer(open(data_file, 'w'))

        c = self.world.topology_manager.topologies[0].cell_manager.newcell(-1,-1)
        self.types = c.types
        self.world.topology_manager.topologies[0].decrement_type_count(c.type)
        c = None

        header = ['#epoch', 'population']
        header += self.types
        self.writer.writerow(header)

    def update(self):
        """Execute the action"""
        if self.skip_update():
	        return

        for top in self.world.topology_manager.topologies:
            row = [self.world.epoch, top.id] + top.typeCount
            self.writer.writerow(row)




# -*- coding: utf-8 -*-
"""
TODO: action description
TODO: make this work for all topologies
"""

__author__ = "TODO"
__credits__ = "TODO"

import csv
import random

from seeds.Action import *

class MovePerson(Action):
    """ TODO: description

        Config file settings:
        [MovePerson]
        epoch_start = 3    Epoch at which to start writing (default 0)
        epoch_end = 100    Epoch at which to stop writing (default end of experiment)
        prob_movement = 0.2 Probability of moving during a given epoch (default 0.5)
        resource = energy  Name of the resource the person deposits
        res_drop_amount = 1 Units of resource dropped per person per update

    """
    def __init__(self, world):
        """Initialize the MovePerson Action"""
        Action.__init__(self, world)

        self.epoch_start = self.world.config.getint('MovePerson', 'epoch_start', 0)
        self.epoch_end = self.world.config.getint('MovePerson', 'epoch_end',
                                                  default=self.world.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.world.config.getint('MovePerson', 'frequency', 1)
        self.prob_movement = self.world.config.getfloat('MovePerson', 'prob_movement', 0.5)
        self.resource = self.world.config.get('MovePerson', 'resource')
        self.res_drop_amount = self.world.config.getfloat('MovePerson', 'res_drop_amount')
        self.name = "MovePerson"

        node = random.choice(self.world.topology_manager.topologies[0].graph.nodes())
        self.cell = self.world.topology_manager.topologies[0].graph.node[node]['cell']

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        if random.random() < self.prob_movement:
            neighbor = random.choice(self.world.topology_manager.topologies[0].get_neighbors(self.cell.node))
            print "Person Moving from Cell %d to %d" % (self.cell.id, neighbor.id)
            self.cell = neighbor

        # TODO deposit resource in node's resource




# -*- coding: utf-8 -*-
"""
Snapshots allow the state of the experiment (World, Cells, Topologies, random
number generator) to be written to disk. This allows the experiment to be
re-started or extended.

It is important to note that open file handles (i.e., files open by an Action)
cannot be saved in the snapshot, so should a experiment be re-started using a
snapshot, new data files will be created.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import pickle
import random
import copy
import time
import bz2

from seeds.ActionManager import *

class SnapshotObj(object):
    """
    SnapshotObjs contain the state of the world and are used by the Snapshot
    class to store these important properties prior to being written to a
    file.

    Properties:
        world
            A reference to the World object, which includes all configuration
            parameters, the Topologies (and therefore Cells and Resources)
        rstate
            The state of the pseudorandom number generator
        timestamp
            The current time
        format_version
            An integer representing the version of this object, should
            this class be added/removed/modified in future revisions.

    """

    def __init__(self):
        """Initialize the SnapshotObj"""
        self.world = None
        self.rstate = None
        self.timestamp = None
        self.format_version = 1

    def update(self, world):
        # Neither copy nor pickle can deal with the csv writer in the stats manager, so
        # back it up, temporarily disable it, copy the world, then restore it
        ambackup = world.action_manager
        pmbackup = world.plugin_manager

        world.action_manager = None
        world.plugin_manager = None
        self.world = copy.deepcopy(world)

        world.action_manager = ambackup
        world.plugin_manager = pmbackup

        self.rstate = random.getstate()
        self.timestamp = time.time()

class Snapshot(object):
    """
    The Snapshot class provides the functionality for creating, updating,
    writing, and restoring the state of experiments.  The intent is to allow
    the state of the world to be saved for analysis or to be used to re-start
    or extend experiments.

    Properties:

    so
        A SnapshotObj used to store the state of the experiment

    """

    def __init__(self):
        """Initialize a Snapshot instance"""
        self.so = SnapshotObj()

    def __str__(self):
        """Produce a string to be used when an Action object is printed"""
        return "Snapshot Object (Created: %s)" % (self.timestamp)

    def update(self, world):
        """Update the Snapshot's view of the world"""
        self.so.update(world)

    def write(self, filename='snapshot'):
        """ Write the snapshot to a file.
        Parameters:

        *filename*
            Base name of the file to be written.  Will result as
            <filename>-<epoch>.snp

        """

        data_file = '%s-%06d.snp' % (filename, self.so.world.epoch)
        outfile = bz2.BZ2File(data_file, 'wb')
        pickle.dump(self.so, outfile)
        outfile.close()

    def read(self, filename='default'):
        """Read the given snapshot file (does not apply it to the experiment)"""
        infile = bz2.BZ2File(filename, 'rb')
        self.so = pickle.load(infile)
        infile.close()

    def apply(self, world):
        """ Apply the current snapshot to the experiment.  This sets the World
        and the state of the pseudorandom number generator.
        
        Actions aren't resumed.  Once this is executed, a new ActionManager is
        initialized.  The data directory will be backed up, and a new one will
        be created.  Any actions currently running that write data to files
        will place the output in the new data directory.

        Similarly, the plugin manager will have to be re-loaded.  This should only
        impact the experiment if plugins used in the initial experiment are not
        available at the time the snapshot is loaded.

        Parameters:

        *world*
            A reference to the World
        
        """

        if self.so.format_version == 1:
            # TODO: not all config values should be transfer.  How to specifiy?
            # - flag in config to say whether or not default has been used.  if not, don't use??

            random.setstate(self.so.rstate)

            world.config = self.so.world.config
            world.epoch = self.so.world.epoch
            world.topology_manager = self.so.world.topology_manager

            world.plugin_manager = PluginManager(world)
            world.action_manager = ActionManager(world)

            world.proceed = True

        else:
            print "Error: Unsupported Snapshot format"





# -*- coding: utf-8 -*-
"""
Handle configurations for experiments

The values stored in a configuration can be read from a file, specified on the
command line or through GUI elements, or set by code elements.  These
configuration values can then be queried or updated.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman"

import ConfigParser
import datetime
import os
import re

import seeds as S

class Config(object):
    """A Config object contains the configuration for an experiment.  The
    values in a configuration can be queried and also updated.
    
    """

    def __init__(self, filename=None):
        """Initialize a Config object

        If an input file is provided, the configuration will be read from this.
        Otherwise, an empty Config is created, and Experiment will need to use
        either default values or values provided through the command line or
        GUI program.

        Named Parameters:

        *filename*
            The name of the configuration to read from

        """
        self.config = ConfigParser.SafeConfigParser()
        self.config.optionxform = str

        if filename != None:
            self.config.read(filename)

        self.resource_sections = []
        for sec in self.config.sections():
            match = re.match("Resource:([a-zA-Z_]+)", sec)
            if match != None:
                self.resource_sections.append(sec)

    def get(self, section, name, default=None):
        """Get a configured value for a variable

        Parameters:

        *section*
            The section under which the variable is defined
        *name*
            The name of the variable
        *default*
            The value to use should the variable not be defined

        """

        try:
            val = self.config.get(section, name)
        except ConfigParser.NoSectionError:
            self.config.add_section(section)
            self.config.set(section, name, str(default))
            val = default
        except ConfigParser.NoOptionError:
            self.config.set(section, name, str(default))
            val = default
        return val

    def getint(self, section, name, default=None):
        """Get a configured integer value for a variable

        Parameters:

        *section*
            The section under which the variable is defined
        *name*
            The name of the variable
        *default*
            The value to use should the variable not be defined

        """

        try:
            val = self.config.getint(section, name)
        except ConfigParser.NoSectionError:
            self.config.add_section(section)
            self.config.set(section, name, str(default))
            val = default
        except ConfigParser.NoOptionError:
            self.config.set(section, name, str(default))
            val = default
        return val

    def getfloat(self, section, name, default=None):
        """Get a configured floating point value for a variable

        Parameters:

        *section*
            The section under which the variable is defined
        *name*
            The name of the variable
        *default*
            The value to use should the variable not be defined

        """

        try:
            val = self.config.getfloat(section, name)
        except ConfigParser.NoSectionError:
            self.config.add_section(section)
            self.config.set(section, name, str(default))
            val = default
        except ConfigParser.NoOptionError:
            self.config.set(section, name, str(default))
            val = default
        return val

    def getboolean(self, section, name, default=None):
        """Get a configured boolean value for a variable

        Parameters:

        *section*
            The section under which the variable is defined
        *name*
            The name of the variable
        *default*
            The value to use should the variable not be defined

        """

        try:
            val = self.config.getboolean(section, name)
        except ConfigParser.NoSectionError:
            self.config.add_section(section)
            self.config.set(section, name, str(default))
            val = default
        except ConfigParser.NoOptionError:
            self.config.set(section, name, str(default))
            val = default
        return val

    def set(self, section, name, value):
        """Set the value for a given variable

        *section*
            The section under which the variable is defined
        *name*
            The name of the variable
        *value*
            The value to set

        """

        val = self.config.set(section, name, str(value))
        return val
        
    def items(self, section):
        """Get a list of (name,value) pairs for all items in a section

        Parameters:

        *section*
            The section from which to retrieve (name,value) pairs

        """

        val = self.config.items(section)
        return val

    def get_resource_sections(self):
        """Get a list of all section names that define a resource
        (e.g., Res:Water)
        
        """

        return self.resource_sections

    def get_resource_names(self):
        """Get a list of the names of all defined resources, (e.g., Water)"""
        resnames = []
        for res in self.get_resource_sections():
            match = re.match("Resource:(?P<resname>[a-zA-Z_]+)", res)
            if match != None:
                resnames.append(match.group("resname"))

        return resnames

    def write(self, filename='experiment.cfg'):
        """Write a file containing the values stored in the Config object.
        This file is written to the configured data directory.

        Parameters:

        *filename*
            The name of the file to create (default: experiment.cfg)

        """

        data_dir = self.get('Experiment', 'data_dir', 'data')
        data_file = os.path.join(data_dir, filename)

        with open(data_file, 'wb') as configfile:
            info = "# SEEDS %s Experiment Configuration\n# Generated: %s\n\n" % (S.__version__, datetime.datetime.now().strftime("%Y-%m-%d (%H:%M:%S)"))
            configfile.write(info)
            self.config.write(configfile)

